@@ -17,28 +17,38 @@
 #include "freertos/FreeRTOS.h"
 #include "freertos/task.h"
 #include "freertos/semphr.h"
-#include "rom/ets_sys.h"
 #include "esp_attr.h"
-#include "esp_intr.h"
-#include "rom/gpio.h"
 #include "soc/spi_reg.h"
 #include "soc/spi_struct.h"
 #include "soc/io_mux_reg.h"
 #include "soc/gpio_sig_map.h"
-#include "soc/dport_reg.h"
 #include "soc/rtc.h"
+#include "driver/periph_ctrl.h"
 
-#define SPI_CLK_IDX(p)  ((p==0)?SPICLK_OUT_IDX:((p==1)?SPICLK_OUT_IDX:((p==2)?HSPICLK_OUT_IDX:((p==3)?VSPICLK_OUT_IDX:0))))
-#define SPI_MISO_IDX(p) ((p==0)?SPIQ_OUT_IDX:((p==1)?SPIQ_OUT_IDX:((p==2)?HSPIQ_OUT_IDX:((p==3)?VSPIQ_OUT_IDX:0))))
-#define SPI_MOSI_IDX(p) ((p==0)?SPID_IN_IDX:((p==1)?SPID_IN_IDX:((p==2)?HSPID_IN_IDX:((p==3)?VSPID_IN_IDX:0))))
-
-#define SPI_SPI_SS_IDX(n)   ((n==0)?SPICS0_OUT_IDX:((n==1)?SPICS1_OUT_IDX:((n==2)?SPICS2_OUT_IDX:SPICS0_OUT_IDX)))
-#define SPI_HSPI_SS_IDX(n)   ((n==0)?HSPICS0_OUT_IDX:((n==1)?HSPICS1_OUT_IDX:((n==2)?HSPICS2_OUT_IDX:HSPICS0_OUT_IDX)))
-#define SPI_VSPI_SS_IDX(n)   ((n==0)?VSPICS0_OUT_IDX:((n==1)?VSPICS1_OUT_IDX:((n==2)?VSPICS2_OUT_IDX:VSPICS0_OUT_IDX)))
-#define SPI_SS_IDX(p, n)   ((p==0)?SPI_SPI_SS_IDX(n):((p==1)?SPI_SPI_SS_IDX(n):((p==2)?SPI_HSPI_SS_IDX(n):((p==3)?SPI_VSPI_SS_IDX(n):0))))
-
-#define SPI_INUM(u)        (2)
-#define SPI_INTR_SOURCE(u) ((u==0)?ETS_SPI0_INTR_SOURCE:((u==1)?ETS_SPI1_INTR_SOURCE:((u==2)?ETS_SPI2_INTR_SOURCE:((p==3)?ETS_SPI3_INTR_SOURCE:0))))
+#include "esp_system.h"
+#ifdef ESP_IDF_VERSION_MAJOR // IDF 4+
+#if CONFIG_IDF_TARGET_ESP32 // ESP32/PICO-D4
+#include "soc/dport_reg.h"
+#include "esp32/rom/ets_sys.h"
+#include "esp32/rom/gpio.h"
+#include "esp_intr_alloc.h"
+#elif CONFIG_IDF_TARGET_ESP32S2
+#include "soc/dport_reg.h"
+#include "esp32s2/rom/ets_sys.h"
+#include "esp32s2/rom/gpio.h"
+#include "esp_intr_alloc.h"
+#elif CONFIG_IDF_TARGET_ESP32C3
+#include "esp32c3/rom/ets_sys.h"
+#include "esp32c3/rom/gpio.h"
+#include "esp_intr_alloc.h"
+#else 
+#error Target CONFIG_IDF_TARGET is not supported
+#endif
+#else // ESP32 Before IDF 4.0
+#include "rom/ets_sys.h"
+#include "rom/gpio.h"
+#include "esp_intr.h"
+#endif
 
 struct spi_struct_t {
     spi_dev_t * dev;
@@ -48,25 +58,84 @@ struct spi_struct_t {
     uint8_t num;
 };
 
+#if CONFIG_IDF_TARGET_ESP32S2
+// ESP32S2
+#define SPI_COUNT           (3)
+
+#define SPI_CLK_IDX(p)      ((p==0)?SPICLK_OUT_MUX_IDX:((p==1)?FSPICLK_OUT_MUX_IDX:((p==2)?SPI3_CLK_OUT_MUX_IDX:0)))
+#define SPI_MISO_IDX(p)     ((p==0)?SPIQ_OUT_IDX:((p==1)?FSPIQ_OUT_IDX:((p==2)?SPI3_Q_OUT_IDX:0)))
+#define SPI_MOSI_IDX(p)     ((p==0)?SPID_IN_IDX:((p==1)?FSPID_IN_IDX:((p==2)?SPI3_D_IN_IDX:0)))
+
+#define SPI_SPI_SS_IDX(n)   ((n==0)?SPICS0_OUT_IDX:((n==1)?SPICS1_OUT_IDX:0))
+#define SPI_HSPI_SS_IDX(n)  ((n==0)?SPI3_CS0_OUT_IDX:((n==1)?SPI3_CS1_OUT_IDX:((n==2)?SPI3_CS2_OUT_IDX:SPI3_CS0_OUT_IDX)))
+#define SPI_FSPI_SS_IDX(n)  ((n==0)?FSPICS0_OUT_IDX:((n==1)?FSPICS1_OUT_IDX:((n==2)?FSPICS2_OUT_IDX:FSPICS0_OUT_IDX)))
+#define SPI_SS_IDX(p, n)    ((p==0)?SPI_SPI_SS_IDX(n):((p==1)?SPI_SPI_SS_IDX(n):((p==2)?SPI_HSPI_SS_IDX(n):0)))
+
+#define SPI_INTR_SOURCE(u)  ((u==0)?ETS_SPI1_INTR_SOURCE:((u==1)?ETS_SPI2_INTR_SOURCE:((u==2)?ETS_SPI3_INTR_SOURCE:0)))
+
+#elif CONFIG_IDF_TARGET_ESP32C3
+// ESP32S2
+#define SPI_COUNT           (1)
+
+#define SPI_CLK_IDX(p)      FSPICLK_OUT_IDX
+#define SPI_MISO_IDX(p)     FSPIQ_OUT_IDX
+#define SPI_MOSI_IDX(p)     FSPID_IN_IDX
+
+#define SPI_SPI_SS_IDX(n)   ((n==0)?FSPICS0_OUT_IDX:((n==1)?FSPICS1_OUT_IDX:((n==2)?FSPICS2_OUT_IDX:FSPICS0_OUT_IDX)))
+#define SPI_SS_IDX(p, n)    SPI_SPI_SS_IDX(n)
+
+#define SPI_INTR_SOURCE(u)  ETS_SPI2_INTR_SOURCE
+
+#else
+// ESP32
+#define SPI_COUNT           (4)
+
+#define SPI_CLK_IDX(p)      ((p==0)?SPICLK_OUT_IDX:((p==1)?SPICLK_OUT_IDX:((p==2)?HSPICLK_OUT_IDX:((p==3)?VSPICLK_OUT_IDX:0))))
+#define SPI_MISO_IDX(p)     ((p==0)?SPIQ_OUT_IDX:((p==1)?SPIQ_OUT_IDX:((p==2)?HSPIQ_OUT_IDX:((p==3)?VSPIQ_OUT_IDX:0))))
+#define SPI_MOSI_IDX(p)     ((p==0)?SPID_IN_IDX:((p==1)?SPID_IN_IDX:((p==2)?HSPID_IN_IDX:((p==3)?VSPID_IN_IDX:0))))
+
+#define SPI_SPI_SS_IDX(n)   ((n==0)?SPICS0_OUT_IDX:((n==1)?SPICS1_OUT_IDX:((n==2)?SPICS2_OUT_IDX:SPICS0_OUT_IDX)))
+#define SPI_HSPI_SS_IDX(n)  ((n==0)?HSPICS0_OUT_IDX:((n==1)?HSPICS1_OUT_IDX:((n==2)?HSPICS2_OUT_IDX:HSPICS0_OUT_IDX)))
+#define SPI_VSPI_SS_IDX(n)  ((n==0)?VSPICS0_OUT_IDX:((n==1)?VSPICS1_OUT_IDX:((n==2)?VSPICS2_OUT_IDX:VSPICS0_OUT_IDX)))
+#define SPI_SS_IDX(p, n)    ((p==0)?SPI_SPI_SS_IDX(n):((p==1)?SPI_SPI_SS_IDX(n):((p==2)?SPI_HSPI_SS_IDX(n):((p==3)?SPI_VSPI_SS_IDX(n):0))))
+
+#define SPI_INTR_SOURCE(u)  ((u==0)?ETS_SPI0_INTR_SOURCE:((u==1)?ETS_SPI1_INTR_SOURCE:((u==2)?ETS_SPI2_INTR_SOURCE:((p==3)?ETS_SPI3_INTR_SOURCE:0))))
+
+#endif
+
 #if CONFIG_DISABLE_HAL_LOCKS
 #define SPI_MUTEX_LOCK()
 #define SPI_MUTEX_UNLOCK()
 
-static spi_t _spi_bus_array[4] = {
+static spi_t _spi_bus_array[] = {
+#if CONFIG_IDF_TARGET_ESP32S2
+    {(volatile spi_dev_t *)(DR_REG_SPI1_BASE), 0},
+    {(volatile spi_dev_t *)(DR_REG_SPI2_BASE), 1},
+    {(volatile spi_dev_t *)(DR_REG_SPI3_BASE), 2}
+#else
     {(volatile spi_dev_t *)(DR_REG_SPI0_BASE), 0},
     {(volatile spi_dev_t *)(DR_REG_SPI1_BASE), 1},
     {(volatile spi_dev_t *)(DR_REG_SPI2_BASE), 2},
     {(volatile spi_dev_t *)(DR_REG_SPI3_BASE), 3}
+#endif
 };
 #else
 #define SPI_MUTEX_LOCK()    do {} while (xSemaphoreTake(spi->lock, portMAX_DELAY) != pdPASS)
 #define SPI_MUTEX_UNLOCK()  xSemaphoreGive(spi->lock)
 
-static spi_t _spi_bus_array[4] = {
+static spi_t _spi_bus_array[] = {
+#if CONFIG_IDF_TARGET_ESP32S2
+    {(volatile spi_dev_t *)(DR_REG_SPI1_BASE), NULL, 0},
+    {(volatile spi_dev_t *)(DR_REG_SPI2_BASE), NULL, 1},
+    {(volatile spi_dev_t *)(DR_REG_SPI3_BASE), NULL, 2}
+#elif CONFIG_IDF_TARGET_ESP32C3
+    {(volatile spi_dev_t *)(&GPSPI2), NULL, FSPI}
+#else
     {(volatile spi_dev_t *)(DR_REG_SPI0_BASE), NULL, 0},
     {(volatile spi_dev_t *)(DR_REG_SPI1_BASE), NULL, 1},
     {(volatile spi_dev_t *)(DR_REG_SPI2_BASE), NULL, 2},
     {(volatile spi_dev_t *)(DR_REG_SPI3_BASE), NULL, 3}
+#endif
 };
 #endif
 
@@ -76,6 +145,14 @@ void spiAttachSCK(spi_t * spi, int8_t sck)
         return;
     }
     if(sck < 0) {
+#if CONFIG_IDF_TARGET_ESP32S2
+        if(spi->num == FSPI) {
+            sck = 36;
+        } else {
+            log_e("HSPI Does not have default pins on ESP32S2!");
+            return;
+        }
+#elif CONFIG_IDF_TARGET_ESP32
         if(spi->num == HSPI) {
             sck = 14;
         } else if(spi->num == VSPI) {
@@ -83,6 +160,10 @@ void spiAttachSCK(spi_t * spi, int8_t sck)
         } else {
             sck = 6;
         }
+#elif CONFIG_IDF_TARGET_ESP32C3
+        log_e("SPI Does not have default pins on ESP32C3!");
+        return;
+#endif
     }
     pinMode(sck, OUTPUT);
     pinMatrixOutAttach(sck, SPI_CLK_IDX(spi->num), false, false);
@@ -94,6 +175,14 @@ void spiAttachMISO(spi_t * spi, int8_t miso)
         return;
     }
     if(miso < 0) {
+#if CONFIG_IDF_TARGET_ESP32S2
+        if(spi->num == FSPI) {
+            miso = 37;
+        } else {
+            log_e("HSPI Does not have default pins on ESP32S2!");
+            return;
+        }
+#elif CONFIG_IDF_TARGET_ESP32
         if(spi->num == HSPI) {
             miso = 12;
         } else if(spi->num == VSPI) {
@@ -101,6 +190,10 @@ void spiAttachMISO(spi_t * spi, int8_t miso)
         } else {
             miso = 7;
         }
+#elif CONFIG_IDF_TARGET_ESP32C3
+        log_e("SPI Does not have default pins on ESP32C3!");
+        return;
+#endif
     }
     SPI_MUTEX_LOCK();
     pinMode(miso, INPUT);
@@ -114,6 +207,14 @@ void spiAttachMOSI(spi_t * spi, int8_t mosi)
         return;
     }
     if(mosi < 0) {
+#if CONFIG_IDF_TARGET_ESP32S2
+        if(spi->num == FSPI) {
+            mosi = 35;
+        } else {
+            log_e("HSPI Does not have default pins on ESP32S2!");
+            return;
+        }
+#elif CONFIG_IDF_TARGET_ESP32
         if(spi->num == HSPI) {
             mosi = 13;
         } else if(spi->num == VSPI) {
@@ -121,6 +222,10 @@ void spiAttachMOSI(spi_t * spi, int8_t mosi)
         } else {
             mosi = 8;
         }
+#elif CONFIG_IDF_TARGET_ESP32C3
+        log_e("SPI Does not have default pins on ESP32C3!");
+        return;
+#endif
     }
     pinMode(mosi, OUTPUT);
     pinMatrixOutAttach(mosi, SPI_MOSI_IDX(spi->num), false, false);
@@ -132,6 +237,14 @@ void spiDetachSCK(spi_t * spi, int8_t sck)
         return;
     }
     if(sck < 0) {
+#if CONFIG_IDF_TARGET_ESP32S2
+        if(spi->num == FSPI) {
+            sck = 36;
+        } else {
+            log_e("HSPI Does not have default pins on ESP32S2!");
+            return;
+        }
+#elif CONFIG_IDF_TARGET_ESP32
         if(spi->num == HSPI) {
             sck = 14;
         } else if(spi->num == VSPI) {
@@ -139,6 +252,10 @@ void spiDetachSCK(spi_t * spi, int8_t sck)
         } else {
             sck = 6;
         }
+#elif CONFIG_IDF_TARGET_ESP32C3
+        log_e("SPI Does not have default pins on ESP32C3!");
+        return;
+#endif
     }
     pinMatrixOutDetach(sck, false, false);
     pinMode(sck, INPUT);
@@ -150,6 +267,14 @@ void spiDetachMISO(spi_t * spi, int8_t miso)
         return;
     }
     if(miso < 0) {
+#if CONFIG_IDF_TARGET_ESP32S2
+        if(spi->num == FSPI) {
+            miso = 37;
+        } else {
+            log_e("HSPI Does not have default pins on ESP32S2!");
+            return;
+        }
+#elif CONFIG_IDF_TARGET_ESP32
         if(spi->num == HSPI) {
             miso = 12;
         } else if(spi->num == VSPI) {
@@ -157,6 +282,10 @@ void spiDetachMISO(spi_t * spi, int8_t miso)
         } else {
             miso = 7;
         }
+#elif CONFIG_IDF_TARGET_ESP32C3
+        log_e("SPI Does not have default pins on ESP32C3!");
+        return;
+#endif
     }
     pinMatrixInDetach(SPI_MISO_IDX(spi->num), false, false);
     pinMode(miso, INPUT);
@@ -168,6 +297,14 @@ void spiDetachMOSI(spi_t * spi, int8_t mosi)
         return;
     }
     if(mosi < 0) {
+#if CONFIG_IDF_TARGET_ESP32S2
+        if(spi->num == FSPI) {
+            mosi = 35;
+        } else {
+            log_e("HSPI Does not have default pins on ESP32S2!");
+            return;
+        }
+#elif CONFIG_IDF_TARGET_ESP32
         if(spi->num == HSPI) {
             mosi = 13;
         } else if(spi->num == VSPI) {
@@ -175,6 +312,10 @@ void spiDetachMOSI(spi_t * spi, int8_t mosi)
         } else {
             mosi = 8;
         }
+#elif CONFIG_IDF_TARGET_ESP32C3
+        log_e("SPI Does not have default pins on ESP32C3!");
+        return;
+#endif
     }
     pinMatrixOutDetach(mosi, false, false);
     pinMode(mosi, INPUT);
@@ -190,6 +331,14 @@ void spiAttachSS(spi_t * spi, uint8_t cs_num, int8_t ss)
     }
     if(ss < 0) {
         cs_num = 0;
+#if CONFIG_IDF_TARGET_ESP32S2
+        if(spi->num == FSPI) {
+            ss = 34;
+        } else {
+            log_e("HSPI Does not have default pins on ESP32S2!");
+            return;
+        }
+#elif CONFIG_IDF_TARGET_ESP32
         if(spi->num == HSPI) {
             ss = 15;
         } else if(spi->num == VSPI) {
@@ -197,6 +346,10 @@ void spiAttachSS(spi_t * spi, uint8_t cs_num, int8_t ss)
         } else {
             ss = 11;
         }
+#elif CONFIG_IDF_TARGET_ESP32C3
+        log_e("SPI Does not have default pins on ESP32C3!");
+        return;
+#endif
     }
     pinMode(ss, OUTPUT);
     pinMatrixOutAttach(ss, SPI_SS_IDX(spi->num, cs_num), false, false);
@@ -209,6 +362,14 @@ void spiDetachSS(spi_t * spi, int8_t ss)
         return;
     }
     if(ss < 0) {
+#if CONFIG_IDF_TARGET_ESP32S2
+        if(spi->num == FSPI) {
+            ss = 34;
+        } else {
+            log_e("HSPI Does not have default pins on ESP32S2!");
+            return;
+        }
+#elif CONFIG_IDF_TARGET_ESP32
         if(spi->num == HSPI) {
             ss = 15;
         } else if(spi->num == VSPI) {
@@ -216,6 +377,10 @@ void spiDetachSS(spi_t * spi, int8_t ss)
         } else {
             ss = 11;
         }
+#elif CONFIG_IDF_TARGET_ESP32C3
+        log_e("SPI Does not have default pins on ESP32C3!");
+        return;
+#endif
     }
     pinMatrixOutDetach(ss, false, false);
     pinMode(ss, INPUT);
@@ -227,7 +392,11 @@ void spiEnableSSPins(spi_t * spi, uint8_t cs_mask)
         return;
     }
     SPI_MUTEX_LOCK();
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->misc.val &= ~(cs_mask & SPI_CS_MASK_ALL);
+#else
     spi->dev->pin.val &= ~(cs_mask & SPI_CS_MASK_ALL);
+#endif
     SPI_MUTEX_UNLOCK();
 }
 
@@ -237,7 +406,11 @@ void spiDisableSSPins(spi_t * spi, uint8_t cs_mask)
         return;
     }
     SPI_MUTEX_LOCK();
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->misc.val |= (cs_mask & SPI_CS_MASK_ALL);
+#else
     spi->dev->pin.val |= (cs_mask & SPI_CS_MASK_ALL);
+#endif
     SPI_MUTEX_UNLOCK();
 }
 
@@ -269,7 +442,11 @@ void spiSSSet(spi_t * spi)
         return;
     }
     SPI_MUTEX_LOCK();
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->misc.cs_keep_active = 1;
+#else
     spi->dev->pin.cs_keep_active = 1;
+#endif
     SPI_MUTEX_UNLOCK();
 }
 
@@ -279,7 +456,11 @@ void spiSSClear(spi_t * spi)
         return;
     }
     SPI_MUTEX_LOCK();
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->misc.cs_keep_active = 0;
+#else
     spi->dev->pin.cs_keep_active = 0;
+#endif
     SPI_MUTEX_UNLOCK();
 }
 
@@ -306,7 +487,11 @@ uint8_t spiGetDataMode(spi_t * spi)
     if(!spi) {
         return 0;
     }
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+    bool idleEdge = spi->dev->misc.ck_idle_edge;
+#else
     bool idleEdge = spi->dev->pin.ck_idle_edge;
+#endif
     bool outEdge = spi->dev->user.ck_out_edge;
     if(idleEdge) {
         if(outEdge) {
@@ -328,20 +513,36 @@ void spiSetDataMode(spi_t * spi, uint8_t dataMode)
     SPI_MUTEX_LOCK();
     switch (dataMode) {
     case SPI_MODE1:
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->misc.ck_idle_edge = 0;
+#else
         spi->dev->pin.ck_idle_edge = 0;
+#endif
         spi->dev->user.ck_out_edge = 1;
         break;
     case SPI_MODE2:
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->misc.ck_idle_edge = 1;
+#else
         spi->dev->pin.ck_idle_edge = 1;
+#endif
         spi->dev->user.ck_out_edge = 1;
         break;
     case SPI_MODE3:
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->misc.ck_idle_edge = 1;
+#else
         spi->dev->pin.ck_idle_edge = 1;
+#endif
         spi->dev->user.ck_out_edge = 0;
         break;
     case SPI_MODE0:
     default:
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->misc.ck_idle_edge = 0;
+#else
         spi->dev->pin.ck_idle_edge = 0;
+#endif
         spi->dev->user.ck_out_edge = 0;
         break;
     }
@@ -384,28 +585,48 @@ static void _on_apb_change(void * arg, apb_change_ev_t ev_type, uint32_t old_apb
     }
 }
 
+static void spiInitBus(spi_t * spi)
+{
+#ifndef CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->slave.trans_done = 0;
+#endif
+    spi->dev->slave.val = 0;
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->misc.val = 0;
+#else
+    spi->dev->pin.val = 0;
+#endif
+    spi->dev->user.val = 0;
+    spi->dev->user1.val = 0;
+    spi->dev->ctrl.val = 0;
+#ifndef CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->ctrl1.val = 0;
+    spi->dev->ctrl2.val = 0;
+#else
+    spi->dev->clk_gate.val = 0;
+    spi->dev->dma_conf.val = 0;
+    spi->dev->dma_conf.rx_afifo_rst = 1;
+    spi->dev->dma_conf.buf_afifo_rst = 1;
+#endif
+    spi->dev->clock.val = 0;
+}
+
 void spiStopBus(spi_t * spi)
 {
     if(!spi) {
         return;
     }
-    SPI_MUTEX_LOCK();
-    spi->dev->slave.trans_done = 0;
-    spi->dev->slave.slave_mode = 0;
-    spi->dev->pin.val = 0;
-    spi->dev->user.val = 0;
-    spi->dev->user1.val = 0;
-    spi->dev->ctrl.val = 0;
-    spi->dev->ctrl1.val = 0;
-    spi->dev->ctrl2.val = 0;
-    spi->dev->clock.val = 0;
-    SPI_MUTEX_UNLOCK();
+
     removeApbChangeCallback(spi, _on_apb_change);
+
+    SPI_MUTEX_LOCK();
+    spiInitBus(spi);
+    SPI_MUTEX_UNLOCK();
 }
 
 spi_t * spiStartBus(uint8_t spi_num, uint32_t clockDiv, uint8_t dataMode, uint8_t bitOrder)
 {
-    if(spi_num > 3){
+    if(spi_num >= SPI_COUNT){
         return NULL;
     }
 
@@ -420,34 +641,58 @@ spi_t * spiStartBus(uint8_t spi_num, uint32_t clockDiv, uint8_t dataMode, uint8_
     }
 #endif
 
-    if(spi_num == HSPI) {
-        DPORT_SET_PERI_REG_MASK(DPORT_PERIP_CLK_EN_REG, DPORT_SPI_CLK_EN);
-        DPORT_CLEAR_PERI_REG_MASK(DPORT_PERIP_RST_EN_REG, DPORT_SPI_RST);
-    } else if(spi_num == VSPI) {
-        DPORT_SET_PERI_REG_MASK(DPORT_PERIP_CLK_EN_REG, DPORT_SPI_CLK_EN_2);
-        DPORT_CLEAR_PERI_REG_MASK(DPORT_PERIP_RST_EN_REG, DPORT_SPI_RST_2);
+#if CONFIG_IDF_TARGET_ESP32S2
+    if(spi_num == FSPI) {
+        DPORT_SET_PERI_REG_MASK(DPORT_PERIP_CLK_EN_REG, DPORT_SPI2_CLK_EN);
+        DPORT_CLEAR_PERI_REG_MASK(DPORT_PERIP_RST_EN_REG, DPORT_SPI2_RST);
+    } else if(spi_num == HSPI) {
+        DPORT_SET_PERI_REG_MASK(DPORT_PERIP_CLK_EN_REG, DPORT_SPI3_CLK_EN);
+        DPORT_CLEAR_PERI_REG_MASK(DPORT_PERIP_RST_EN_REG, DPORT_SPI3_RST);
     } else {
-        DPORT_SET_PERI_REG_MASK(DPORT_PERIP_CLK_EN_REG, DPORT_SPI_CLK_EN_1);
-        DPORT_CLEAR_PERI_REG_MASK(DPORT_PERIP_RST_EN_REG, DPORT_SPI_RST_1);
+        DPORT_SET_PERI_REG_MASK(DPORT_PERIP_CLK_EN_REG, DPORT_SPI01_CLK_EN);
+        DPORT_CLEAR_PERI_REG_MASK(DPORT_PERIP_RST_EN_REG, DPORT_SPI01_RST);
     }
-
-    spiStopBus(spi);
-    spiSetDataMode(spi, dataMode);
-    spiSetBitOrder(spi, bitOrder);
-    spiSetClockDiv(spi, clockDiv);
+#elif CONFIG_IDF_TARGET_ESP32
+    if(spi_num == HSPI) {
+        DPORT_SET_PERI_REG_MASK(DPORT_PERIP_CLK_EN_REG, DPORT_SPI2_CLK_EN);
+        DPORT_CLEAR_PERI_REG_MASK(DPORT_PERIP_RST_EN_REG, DPORT_SPI2_RST);
+    } else if(spi_num == VSPI) {
+        DPORT_SET_PERI_REG_MASK(DPORT_PERIP_CLK_EN_REG, DPORT_SPI3_CLK_EN);
+        DPORT_CLEAR_PERI_REG_MASK(DPORT_PERIP_RST_EN_REG, DPORT_SPI3_RST);
+    } else {
+        DPORT_SET_PERI_REG_MASK(DPORT_PERIP_CLK_EN_REG, DPORT_SPI01_CLK_EN);
+        DPORT_CLEAR_PERI_REG_MASK(DPORT_PERIP_RST_EN_REG, DPORT_SPI01_RST);
+    }
+#elif CONFIG_IDF_TARGET_ESP32C3
+    periph_module_reset( PERIPH_SPI2_MODULE );
+    periph_module_enable( PERIPH_SPI2_MODULE );
+#endif
 
     SPI_MUTEX_LOCK();
+    spiInitBus(spi);
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->clk_gate.clk_en = 1;
+    spi->dev->clk_gate.mst_clk_sel = 1;
+    spi->dev->clk_gate.mst_clk_active = 1;
+    spi->dev->dma_conf.tx_seg_trans_clr_en = 1;
+    spi->dev->dma_conf.rx_seg_trans_clr_en = 1;
+    spi->dev->dma_conf.dma_seg_trans_en = 0;
+#endif
     spi->dev->user.usr_mosi = 1;
     spi->dev->user.usr_miso = 1;
     spi->dev->user.doutdin = 1;
-
     int i;
     for(i=0; i<16; i++) {
         spi->dev->data_buf[i] = 0x00000000;
     }
     SPI_MUTEX_UNLOCK();
 
+    spiSetDataMode(spi, dataMode);
+    spiSetBitOrder(spi, bitOrder);
+    spiSetClockDiv(spi, clockDiv);
+
     addApbChangeCallback(spi, _on_apb_change);
+
     return spi;
 }
 
@@ -459,7 +704,17 @@ void spiWaitReady(spi_t * spi)
     while(spi->dev->cmd.usr);
 }
 
-void spiWrite(spi_t * spi, uint32_t *data, uint8_t len)
+#if CONFIG_IDF_TARGET_ESP32S2
+#define usr_mosi_dbitlen usr_mosi_bit_len
+#define usr_miso_dbitlen usr_miso_bit_len
+#elif CONFIG_IDF_TARGET_ESP32C3
+#define usr_mosi_dbitlen ms_data_bitlen
+#define usr_miso_dbitlen ms_data_bitlen
+#define mosi_dlen ms_dlen
+#define miso_dlen ms_dlen
+#endif
+
+void spiWrite(spi_t * spi, const uint32_t *data, uint8_t len)
 {
     if(!spi) {
         return;
@@ -470,10 +725,16 @@ void spiWrite(spi_t * spi, uint32_t *data, uint8_t len)
     }
     SPI_MUTEX_LOCK();
     spi->dev->mosi_dlen.usr_mosi_dbitlen = (len * 32) - 1;
+#ifndef CONFIG_IDF_TARGET_ESP32C3
     spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#endif
     for(i=0; i<len; i++) {
         spi->dev->data_buf[i] = data[i];
     }
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     SPI_MUTEX_UNLOCK();
@@ -494,6 +755,10 @@ void spiTransfer(spi_t * spi, uint32_t *data, uint8_t len)
     for(i=0; i<len; i++) {
         spi->dev->data_buf[i] = data[i];
     }
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     for(i=0; i<len; i++) {
@@ -509,8 +774,14 @@ void spiWriteByte(spi_t * spi, uint8_t data)
     }
     SPI_MUTEX_LOCK();
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 7;
+#ifndef CONFIG_IDF_TARGET_ESP32C3
     spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#endif
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     SPI_MUTEX_UNLOCK();
@@ -525,6 +796,10 @@ uint8_t spiTransferByte(spi_t * spi, uint8_t data)
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 7;
     spi->dev->miso_dlen.usr_miso_dbitlen = 7;
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     data = spi->dev->data_buf[0] & 0xFF;
@@ -532,17 +807,7 @@ uint8_t spiTransferByte(spi_t * spi, uint8_t data)
     return data;
 }
 
-uint32_t __spiTranslate24(uint32_t data)
-{
-    union {
-        uint32_t l;
-        uint8_t b[4];
-    } out;
-    out.l = data;
-    return out.b[2] | (out.b[1] << 8) | (out.b[0] << 16);
-}
-
-uint32_t __spiTranslate32(uint32_t data)
+static uint32_t __spiTranslate32(uint32_t data)
 {
     union {
         uint32_t l;
@@ -562,8 +827,14 @@ void spiWriteWord(spi_t * spi, uint16_t data)
     }
     SPI_MUTEX_LOCK();
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 15;
+#ifndef CONFIG_IDF_TARGET_ESP32C3
     spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#endif
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     SPI_MUTEX_UNLOCK();
@@ -581,6 +852,10 @@ uint16_t spiTransferWord(spi_t * spi, uint16_t data)
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 15;
     spi->dev->miso_dlen.usr_miso_dbitlen = 15;
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     data = spi->dev->data_buf[0];
@@ -601,8 +876,14 @@ void spiWriteLong(spi_t * spi, uint32_t data)
     }
     SPI_MUTEX_LOCK();
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 31;
+#ifndef CONFIG_IDF_TARGET_ESP32C3
     spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#endif
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     SPI_MUTEX_UNLOCK();
@@ -620,6 +901,10 @@ uint32_t spiTransferLong(spi_t * spi, uint32_t data)
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 31;
     spi->dev->miso_dlen.usr_miso_dbitlen = 31;
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     data = spi->dev->data_buf[0];
@@ -630,7 +915,7 @@ uint32_t spiTransferLong(spi_t * spi, uint32_t data)
     return data;
 }
 
-void __spiTransferBytes(spi_t * spi, uint8_t * data, uint8_t * out, uint32_t bytes)
+static void __spiTransferBytes(spi_t * spi, const uint8_t * data, uint8_t * out, uint32_t bytes)
 {
     if(!spi) {
         return;
@@ -659,6 +944,10 @@ void __spiTransferBytes(spi_t * spi, uint8_t * data, uint8_t * out, uint32_t byt
         spi->dev->data_buf[i] = wordsBuf[i];    //copy buffer to spi fifo
     }
 
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
 
     while(spi->dev->cmd.usr);
@@ -671,7 +960,7 @@ void __spiTransferBytes(spi_t * spi, uint8_t * data, uint8_t * out, uint32_t byt
     }
 }
 
-void spiTransferBytes(spi_t * spi, uint8_t * data, uint8_t * out, uint32_t size)
+void spiTransferBytes(spi_t * spi, const uint8_t * data, uint8_t * out, uint32_t size)
 {
     if(!spi) {
         return;
@@ -722,23 +1011,39 @@ void spiTransaction(spi_t * spi, uint32_t clockDiv, uint8_t dataMode, uint8_t bi
     SPI_MUTEX_LOCK();
     spi->dev->clock.val = clockDiv;
     switch (dataMode) {
-        case SPI_MODE1:
-            spi->dev->pin.ck_idle_edge = 0;
-            spi->dev->user.ck_out_edge = 1;
-            break;
-        case SPI_MODE2:
-            spi->dev->pin.ck_idle_edge = 1;
-            spi->dev->user.ck_out_edge = 1;
-            break;
-        case SPI_MODE3:
-            spi->dev->pin.ck_idle_edge = 1;
-            spi->dev->user.ck_out_edge = 0;
-            break;
-        case SPI_MODE0:
-        default:
-            spi->dev->pin.ck_idle_edge = 0;
-            spi->dev->user.ck_out_edge = 0;
-            break;
+    case SPI_MODE1:
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->misc.ck_idle_edge = 0;
+#else
+        spi->dev->pin.ck_idle_edge = 0;
+#endif
+        spi->dev->user.ck_out_edge = 1;
+        break;
+    case SPI_MODE2:
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->misc.ck_idle_edge = 1;
+#else
+        spi->dev->pin.ck_idle_edge = 1;
+#endif
+        spi->dev->user.ck_out_edge = 1;
+        break;
+    case SPI_MODE3:
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->misc.ck_idle_edge = 1;
+#else
+        spi->dev->pin.ck_idle_edge = 1;
+#endif
+        spi->dev->user.ck_out_edge = 0;
+        break;
+    case SPI_MODE0:
+    default:
+#if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->misc.ck_idle_edge = 0;
+#else
+        spi->dev->pin.ck_idle_edge = 0;
+#endif
+        spi->dev->user.ck_out_edge = 0;
+        break;
     }
     if (SPI_MSBFIRST == bitOrder) {
         spi->dev->ctrl.wr_bit_order = 0;
@@ -765,14 +1070,20 @@ void spiEndTransaction(spi_t * spi)
     SPI_MUTEX_UNLOCK();
 }
 
-void IRAM_ATTR spiWriteByteNL(spi_t * spi, uint8_t data)
+void ARDUINO_ISR_ATTR spiWriteByteNL(spi_t * spi, uint8_t data)
 {
     if(!spi) {
         return;
     }
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 7;
+#ifndef CONFIG_IDF_TARGET_ESP32C3
     spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#endif
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
 }
@@ -785,13 +1096,17 @@ uint8_t spiTransferByteNL(spi_t * spi, uint8_t data)
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 7;
     spi->dev->miso_dlen.usr_miso_dbitlen = 7;
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     data = spi->dev->data_buf[0] & 0xFF;
     return data;
 }
 
-void IRAM_ATTR spiWriteShortNL(spi_t * spi, uint16_t data)
+void ARDUINO_ISR_ATTR spiWriteShortNL(spi_t * spi, uint16_t data)
 {
     if(!spi) {
         return;
@@ -800,8 +1115,14 @@ void IRAM_ATTR spiWriteShortNL(spi_t * spi, uint16_t data)
         MSB_16_SET(data, data);
     }
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 15;
+#ifndef CONFIG_IDF_TARGET_ESP32C3
     spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#endif
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
 }
@@ -817,6 +1138,10 @@ uint16_t spiTransferShortNL(spi_t * spi, uint16_t data)
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 15;
     spi->dev->miso_dlen.usr_miso_dbitlen = 15;
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     data = spi->dev->data_buf[0] & 0xFFFF;
@@ -826,7 +1151,7 @@ uint16_t spiTransferShortNL(spi_t * spi, uint16_t data)
     return data;
 }
 
-void IRAM_ATTR spiWriteLongNL(spi_t * spi, uint32_t data)
+void ARDUINO_ISR_ATTR spiWriteLongNL(spi_t * spi, uint32_t data)
 {
     if(!spi) {
         return;
@@ -835,8 +1160,14 @@ void IRAM_ATTR spiWriteLongNL(spi_t * spi, uint32_t data)
         MSB_32_SET(data, data);
     }
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 31;
+#ifndef CONFIG_IDF_TARGET_ESP32C3
     spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#endif
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
 }
@@ -852,6 +1183,10 @@ uint32_t spiTransferLongNL(spi_t * spi, uint32_t data)
     spi->dev->mosi_dlen.usr_mosi_dbitlen = 31;
     spi->dev->miso_dlen.usr_miso_dbitlen = 31;
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     data = spi->dev->data_buf[0];
@@ -861,7 +1196,10 @@ uint32_t spiTransferLongNL(spi_t * spi, uint32_t data)
     return data;
 }
 
-void spiWriteNL(spi_t * spi, const void * data_in, size_t len){
+void spiWriteNL(spi_t * spi, const void * data_in, uint32_t len){
+    if(!spi) {
+        return;
+    }
     size_t longs = len >> 2;
     if(len & 3){
         longs++;
@@ -874,10 +1212,16 @@ void spiWriteNL(spi_t * spi, const void * data_in, size_t len){
         c_longs = (longs > 16)?16:longs;
 
         spi->dev->mosi_dlen.usr_mosi_dbitlen = (c_len*8)-1;
-        spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#ifndef CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#endif
         for (int i=0; i<c_longs; i++) {
             spi->dev->data_buf[i] = data[i];
         }
+#if CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->cmd.update = 1;
+        while (spi->dev->cmd.update);
+#endif
         spi->dev->cmd.usr = 1;
         while(spi->dev->cmd.usr);
 
@@ -887,7 +1231,7 @@ void spiWriteNL(spi_t * spi, const void * data_in, size_t len){
     }
 }
 
-void spiTransferBytesNL(spi_t * spi, const void * data_in, uint8_t * data_out, size_t len){
+void spiTransferBytesNL(spi_t * spi, const void * data_in, uint8_t * data_out, uint32_t len){
     if(!spi) {
         return;
     }
@@ -914,11 +1258,27 @@ void spiTransferBytesNL(spi_t * spi, const void * data_in, uint8_t * data_out, s
                 spi->dev->data_buf[i] = 0xFFFFFFFF;
             }
         }
+#if CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->cmd.update = 1;
+        while (spi->dev->cmd.update);
+#endif
         spi->dev->cmd.usr = 1;
         while(spi->dev->cmd.usr);
         if(result){
-            for (int i=0; i<c_longs; i++) {
-                result[i] = spi->dev->data_buf[i];
+            if(c_len & 3){
+                for (int i=0; i<(c_longs-1); i++) {
+                    result[i] = spi->dev->data_buf[i];
+                }
+                uint32_t last_data = spi->dev->data_buf[c_longs-1];
+                uint8_t * last_out8 = (uint8_t *)&result[c_longs-1];
+                uint8_t * last_data8 = (uint8_t *)&last_data;
+                for (int i=0; i<(c_len & 3); i++) {
+                    last_out8[i] = last_data8[i];
+                }
+            } else {
+                for (int i=0; i<c_longs; i++) {
+                    result[i] = spi->dev->data_buf[i];
+                }
             }
         }
         if(data){
@@ -957,6 +1317,10 @@ void spiTransferBitsNL(spi_t * spi, uint32_t data, uint32_t * out, uint8_t bits)
     spi->dev->mosi_dlen.usr_mosi_dbitlen = (bits - 1);
     spi->dev->miso_dlen.usr_miso_dbitlen = (bits - 1);
     spi->dev->data_buf[0] = data;
+#if CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->cmd.update = 1;
+    while (spi->dev->cmd.update);
+#endif
     spi->dev->cmd.usr = 1;
     while(spi->dev->cmd.usr);
     data = spi->dev->data_buf[0];
@@ -974,7 +1338,7 @@ void spiTransferBitsNL(spi_t * spi, uint32_t data, uint32_t * out, uint8_t bits)
     }
 }
 
-void IRAM_ATTR spiWritePixelsNL(spi_t * spi, const void * data_in, size_t len){
+void ARDUINO_ISR_ATTR spiWritePixelsNL(spi_t * spi, const void * data_in, uint32_t len){
     size_t longs = len >> 2;
     if(len & 3){
         longs++;
@@ -989,7 +1353,9 @@ void IRAM_ATTR spiWritePixelsNL(spi_t * spi, const void * data_in, size_t len){
         l_bytes = (c_len & 3);
 
         spi->dev->mosi_dlen.usr_mosi_dbitlen = (c_len*8)-1;
-        spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#ifndef CONFIG_IDF_TARGET_ESP32C3
+    spi->dev->miso_dlen.usr_miso_dbitlen = 0;
+#endif
         for (int i=0; i<c_longs; i++) {
             if(msb){
                 if(l_bytes && i == (c_longs - 1)){
@@ -1005,6 +1371,10 @@ void IRAM_ATTR spiWritePixelsNL(spi_t * spi, const void * data_in, size_t len){
                 spi->dev->data_buf[i] = data[i];
             }
         }
+#if CONFIG_IDF_TARGET_ESP32C3
+        spi->dev->cmd.update = 1;
+        while (spi->dev->cmd.update);
+#endif
         spi->dev->cmd.usr = 1;
         while(spi->dev->cmd.usr);
 
@@ -1027,7 +1397,12 @@ typedef union {
             uint32_t clkcnt_l:       6;                     /*it must be equal to spi_clkcnt_N.*/
             uint32_t clkcnt_h:       6;                     /*it must be floor((spi_clkcnt_N+1)/2-1).*/
             uint32_t clkcnt_n:       6;                     /*it is the divider of spi_clk. So spi_clk frequency is system/(spi_clkdiv_pre+1)/(spi_clkcnt_N+1)*/
+#if CONFIG_IDF_TARGET_ESP32C3
+            uint32_t clkdiv_pre:     4;                     /*it is pre-divider of spi_clk.*/
+            uint32_t reserved:       9;  					/*reserved*/
+#else
             uint32_t clkdiv_pre:    13;                     /*it is pre-divider of spi_clk.*/
+#endif
             uint32_t clk_equ_sysclk: 1;                     /*1: spi_clk is eqaul to system 0: spi_clk is divided from system clock.*/
     };
 } spiClk_t;
@@ -1069,8 +1444,13 @@ uint32_t spiFrequencyToClockDiv(uint32_t freq)
 
         while(calPreVari++ <= 1) {
             calPre = (((apb_freq / (reg.clkcnt_n + 1)) / freq) - 1) + calPreVari;
+#if CONFIG_IDF_TARGET_ESP32C3
+            if(calPre > 0xF) {
+                reg.clkdiv_pre = 0xF;
+#else
             if(calPre > 0x1FFF) {
                 reg.clkdiv_pre = 0x1FFF;
+#endif
             } else if(calPre <= 0) {
                 reg.clkdiv_pre = 0;
             } else {
