@@ -28,12 +28,14 @@ protected:
 
 public:
     SDFS(FSImplPtr impl);
-    bool begin(uint8_t ssPin=SS, SPIClass &spi=SPI, uint32_t frequency=4000000, const char * mountpoint="/sd", uint8_t max_files=5);
+    bool begin(uint8_t ssPin=SS, SPIClass &spi=SPI, uint32_t frequency=4000000, const char * mountpoint="/sd", uint8_t max_files=5, bool format_if_empty=false);
     void end();
     sdcard_type_t cardType();
     uint64_t cardSize();
     uint64_t totalBytes();
     uint64_t usedBytes();
+    bool readRAW(uint8_t* buffer, uint32_t sector);
+    bool writeRAW(uint8_t* buffer, uint32_t sector);
 };
 
 }
