@@ -12,10 +12,13 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 #include "sd_diskio.h"
+#include "esp_system.h"
 extern "C" {
-    #include "diskio.h"
-    #include "ffconf.h"
     #include "ff.h"
+    #include "diskio.h"
+#if ESP_IDF_VERSION_MAJOR > 3
+    #include "diskio_impl.h"
+#endif
     //#include "esp_vfs.h"
     #include "esp_vfs_fat.h"
     char CRC7(const char* data, int length);
@@ -59,6 +62,31 @@ typedef struct {
 
 static ardu_sdcard_t* s_cards[FF_VOLUMES] = { NULL };
 
+#if ARDUHAL_LOG_LEVEL >= ARDUHAL_LOG_LEVEL_ERROR
+const char * fferr2str[] = {
+    "(0) Succeeded",
+    "(1) A hard error occurred in the low level disk I/O layer",
+    "(2) Assertion failed",
+    "(3) The physical drive cannot work",
+    "(4) Could not find the file",
+    "(5) Could not find the path",
+    "(6) The path name format is invalid",
+    "(7) Access denied due to prohibited access or directory full",
+    "(8) Access denied due to prohibited access",
+    "(9) The file/directory object is invalid",
+    "(10) The physical drive is write protected",
+    "(11) The logical drive number is invalid",
+    "(12) The volume has no work area",
+    "(13) There is no valid FAT volume",
+    "(14) The f_mkfs() aborted due to any problem",
+    "(15) Could not get a grant to access the volume within defined period",
+    "(16) The operation is rejected according to the file sharing policy",
+    "(17) LFN working buffer could not be allocated",
+    "(18) Number of open files > FF_FS_LOCK",
+    "(19) Given parameter is invalid"
+};
+#endif
+
 /*
  * SD SPI
  * */
@@ -72,6 +100,9 @@ bool sdWait(uint8_t pdrv, int timeout)
         resp = s_cards[pdrv]->spi->transfer(0xFF);
     } while (resp == 0x00 && (millis() - start) < (unsigned int)timeout);
 
+    if (!resp) {
+        log_w("Wait Failed");
+    }
     return (resp > 0x00);
 }
 
@@ -90,7 +121,12 @@ bool sdSelectCard(uint8_t pdrv)
 {
     ardu_sdcard_t * card = s_cards[pdrv];
     digitalWrite(card->ssPin, LOW);
-    sdWait(pdrv, 300);
+    bool s = sdWait(pdrv, 300);
+    if (!s) {
+        log_e("Select Failed");
+        digitalWrite(card->ssPin, HIGH);
+        return false;
+    }
     return true;
 }
 
@@ -104,10 +140,11 @@ char sdCommand(uint8_t pdrv, char cmd, unsigned int arg, unsigned int* resp)
             token = sdCommand(pdrv, APP_CMD, 0, NULL);
             sdDeselectCard(pdrv);
             if (token > 1) {
-                return token;
+                break;
             }
             if(!sdSelectCard(pdrv)) {
-                return 0xFF;
+                token = 0xFF;
+                break;
             }
         }
 
@@ -158,7 +195,10 @@ char sdCommand(uint8_t pdrv, char cmd, unsigned int arg, unsigned int* resp)
 
         break;
     }
-
+    if (token == 0xFF) {
+        log_e("Card Failed! cmd: 0x%02x", cmd);
+        card->status = STA_NOINIT;
+    }
     return token;
 }
 
@@ -214,7 +254,7 @@ bool sdReadSector(uint8_t pdrv, char* buffer, unsigned long long sector)
 {
     for (int f = 0; f < 3; f++) {
         if(!sdSelectCard(pdrv)) {
-            break;
+            return false;
         }
         if (!sdCommand(pdrv, READ_BLOCK_SINGLE, (s_cards[pdrv]->type == CARD_SDHC) ? sector : sector << 9, NULL)) {
             bool success = sdReadBytes(pdrv, buffer, 512);
@@ -234,7 +274,7 @@ bool sdReadSectors(uint8_t pdrv, char* buffer, unsigned long long sector, int co
 {
     for (int f = 0; f < 3;) {
         if(!sdSelectCard(pdrv)) {
-            break;
+            return false;
         }
 
         if (!sdCommand(pdrv, READ_BLOCK_MULTIPLE, (s_cards[pdrv]->type == CARD_SDHC) ? sector : sector << 9, NULL)) {
@@ -270,7 +310,7 @@ bool sdWriteSector(uint8_t pdrv, const char* buffer, unsigned long long sector)
 {
     for (int f = 0; f < 3; f++) {
         if(!sdSelectCard(pdrv)) {
-            break;
+            return false;
         }
         if (!sdCommand(pdrv, WRITE_BLOCK_SINGLE, (s_cards[pdrv]->type == CARD_SDHC) ? sector : sector << 9, NULL)) {
             char token = sdWriteBytes(pdrv, buffer, 0xFE);
@@ -306,12 +346,12 @@ bool sdWriteSectors(uint8_t pdrv, const char* buffer, unsigned long long sector,
     for (int f = 0; f < 3;) {
         if (card->type != CARD_MMC) {
             if (sdTransaction(pdrv, SET_WR_BLK_ERASE_COUNT, currentCount, NULL)) {
-                break;
+                return false;
             }
         }
 
         if(!sdSelectCard(pdrv)) {
-            break;
+            return false;
         }
 
         if (!sdCommand(pdrv, WRITE_BLOCK_MULTIPLE, (card->type == CARD_SDHC) ? currentSector : currentSector << 9, NULL)) {
@@ -343,9 +383,8 @@ bool sdWriteSectors(uint8_t pdrv, const char* buffer, unsigned long long sector,
                     break;
                 }
 
-                sdDeselectCard(pdrv);
-
                 if (token == 0x0A) {
+                    sdDeselectCard(pdrv);
                     unsigned int writtenBlocks = 0;
                     if (card->type != CARD_MMC && sdSelectCard(pdrv)) {
                         if (!sdCommand(pdrv, SEND_NUM_WR_BLOCKS, 0, NULL)) {
@@ -364,7 +403,7 @@ bool sdWriteSectors(uint8_t pdrv, const char* buffer, unsigned long long sector,
                     currentCount = count - writtenBlocks;
                     continue;
                 } else {
-                    return false;
+                    break;
                 }
             }
         } else {
@@ -379,7 +418,7 @@ unsigned long sdGetSectorsCount(uint8_t pdrv)
 {
     for (int f = 0; f < 3; f++) {
         if(!sdSelectCard(pdrv)) {
-            break;
+            return false;
         }
 
         if (!sdCommand(pdrv, SEND_CSD, 0, NULL)) {
@@ -607,8 +646,9 @@ DRESULT ff_sd_write(uint8_t pdrv, const uint8_t* buffer, DWORD sector, UINT coun
 
     if (count > 1) {
         res = sdWriteSectors(pdrv, (const char*)buffer, sector, count) ? RES_OK : RES_ERROR;
+    } else {
+        res = sdWriteSector(pdrv, (const char*)buffer, sector) ? RES_OK : RES_ERROR;
     }
-    res = sdWriteSector(pdrv, (const char*)buffer, sector) ? RES_OK : RES_ERROR;
     return res;
 }
 
@@ -637,6 +677,15 @@ DRESULT ff_sd_ioctl(uint8_t pdrv, uint8_t cmd, void* buff)
     return RES_PARERR;
 }
 
+bool sd_read_raw(uint8_t pdrv, uint8_t* buffer, DWORD sector)
+{
+    return ff_sd_read(pdrv, buffer, sector, 1) == ESP_OK;
+}
+
+bool sd_write_raw(uint8_t pdrv, uint8_t* buffer, DWORD sector)
+{
+    return ff_sd_write(pdrv, buffer, sector, 1) == ESP_OK;
+}
 
 /*
  * Public methods
@@ -648,11 +697,13 @@ uint8_t sdcard_uninit(uint8_t pdrv)
     if (pdrv >= FF_VOLUMES || card == NULL) {
         return 1;
     }
+    sdTransaction(pdrv, GO_IDLE_STATE, 0, NULL);
     ff_diskio_register(pdrv, NULL);
     s_cards[pdrv] = NULL;
     esp_err_t err = ESP_OK;
     if (card->base_path) {
         err = esp_vfs_fat_unregister_path(card->base_path);
+        free(card->base_path);
     }
     free(card);
     return err;
@@ -711,7 +762,7 @@ uint8_t sdcard_unmount(uint8_t pdrv)
     return 0;
 }
 
-bool sdcard_mount(uint8_t pdrv, const char* path, uint8_t max_files)
+bool sdcard_mount(uint8_t pdrv, const char* path, uint8_t max_files, bool format_if_empty)
 {
     ardu_sdcard_t * card = s_cards[pdrv];
     if(pdrv >= FF_VOLUMES || card == NULL){
@@ -736,9 +787,25 @@ bool sdcard_mount(uint8_t pdrv, const char* path, uint8_t max_files)
 
     FRESULT res = f_mount(fs, drv, 1);
     if (res != FR_OK) {
-        log_e("f_mount failed 0x(%x)", res);
-        esp_vfs_fat_unregister_path(path);
-        return false;
+        log_e("f_mount failed: %s", fferr2str[res]);
+        if(res == 13 && format_if_empty){
+            BYTE work[FF_MAX_SS];
+            res = f_mkfs(drv, FM_ANY, 0, work, sizeof(work));
+            if (res != FR_OK) {
+                log_e("f_mkfs failed: %s", fferr2str[res]);
+                esp_vfs_fat_unregister_path(path);
+                return false;
+            }
+            res = f_mount(fs, drv, 1);
+            if (res != FR_OK) {
+                log_e("f_mount failed: %s", fferr2str[res]);
+                esp_vfs_fat_unregister_path(path);
+                return false;
+            }
+        } else {
+            esp_vfs_fat_unregister_path(path);
+            return false;
+        }
     }
     AcquireSPI lock(card);
     card->sectors = sdGetSectorsCount(pdrv);
