@@ -12,22 +12,22 @@
 #define digitalPinHasPWM(p)         (p < 34)
 
 // touch screen
-#define TP_SDA              14
-#define TP_SCL              15
+#define TP_SDA              23
+#define TP_SCL              32
 #define TP_INT              38
 
 // Interrupt IO port
 #define RTC_INT             37
 #define APX20X_INT          35
 #define BMA42X_INT1         39
-#define BMA42X_INT2         4
+
+static const uint8_t TX = 1;
+static const uint8_t RX = 3;
 
 //Serial1 Already assigned to GPS LORA
 #define TX1                 33
 #define RX1                 34
 
-static const uint8_t KEY_BUILTIN = 36;
-
 // Already assigned to BMA423 PCF8563 and external extensions
 static const uint8_t SDA = 21;
 static const uint8_t SCL = 22;
